class SocialAuthConf:
    # Social authentication
    SOCIAL_AUTH_LOGIN_REDIRECT_URL = "/"
    SOCIAL_AUTH_LOGIN_URL = "/auth/login/"

    # OAUTH keys (not working yet)
    SOCIAL_AUTH_GITHUB_KEY = ""
    SOCIAL_AUTH_GITHUB_SECRET = ""

    SOCIAL_AUTH_GOOGLE_OAUTH2_KEY = ""
    SOCIAL_AUTH_GOOGLE_OAUTH2_SECRET = ""
    SOCIAL_AUTH_GOOGLE_OAUTH2_SCOPE = ""

    SOCIAL_AUTH_FACEBOOK_KEY = "1085127354890672"
    SOCIAL_AUTH_FACEBOOK_SECRET = "9f6ba5c8721172acaf25a733c4c81a99"
    SOCIAL_AUTH_FACEBOOK_SCOPE = ["email"]

    SOCIAL_AUTH_TWITTER_KEY = "JrIVXbXZguPeUsnqcjtbQEWXH"
    SOCIAL_AUTH_TWITTER_SECRET = "vMZjdO7DsUV8mVo46smQK2SHyhCxnXyc24gxH6J6cH08anWqHA"
